/****************************************************************************

 MESSAGE PASSING INTERFACE TEST CASE SUITE

 Copyright IBM Corp. 1995

 IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 distribute this software for any purpose and without fee provided that the
 above copyright notice and the following paragraphs appear in all copies.

 IBM Corp. makes no representation that the test cases comprising this
 suite are correct or are an accurate representation of any standard.

 In no event shall IBM be liable to any party for direct, indirect, special
 incidental, or consequential damage arising out of the use of this software
 even if IBM Corp. has been advised of the possibility of such damage.

 IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

****************************************************************************

 These test cases reflect an interpretation of the MPI Standard.  They are
 are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 test case from this set believes that the MPI Standard requires behavior
 different than that implied by the test case we would appreciate feedback.

 Comments may be sent to:
    Richard Treumann
    treumann@kgn.ibm.com


****************************************************************************

 MPI-Java version :
    Sung-Hoon Ko(shko@npac.syr.edu)
    Northeast Parallel Architectures Center at Syracuse University
    09/10/99

****************************************************************************
*/

import mpi.*;
 
class free {
    static public void main(String[] args) {

	final int ITER=100;
	int i,me,rc,class;
	Status status;
	Prequest request;
	Comm comm;
	
	int datain[] = new int[1];
	int dataout[] = new int[1];

	
	MPI.Init(args);

	comm = MPI.COMM_WORLD;
	me = comm.Rank();

	if(me < 2)  {
	    if(me == 0)  {
		for(i=0;i<ITER;i++)  {
		    dataout[0] = i;
		    //MPI_Isend(&dataout,1,MPI_INT,1,0,comm,&request);
		    request = comm.Isend(data,0,1,MPI.INT,1,0);
		    //MPI_Request_free(&request);
		    request.Free();

		    //if(request != MPI_REQUEST_NULL) 
		    if(!request.Is_null()) 
			System.out.println("ERROR in MPI_Request_free, request not set to NULL");
		    //MPI_Recv(&datain,1,MPI_INT,1,0,comm,&status);
		    status = comm.Recv(datain,0,1,MPI.INT,1,0);
		    if(datain[0] != i)  
			System.out.println("ERROR in MPI_Recv, datain[0] = "+datain[0]+", should be "+i); 
		}
	    } else if(me == 1) {
		//MPI_Recv(&datain,1,MPI_INT,0,0,comm,&status);
		status = comm.Recv(datain,0,1,MPI.INT,0,0);
		for(i=0;i<ITER-1;i++)  {
		    dataout[0] = i;
		    //MPI_Isend(&dataout,1,MPI_INT,0,0,comm,&request);
		    request =  comm.Isend(dataout,0,1,MPI.INT,0,0);
		    //MPI_Request_free(&request);
		    request.Free();
		    //if(request != MPI_REQUEST_NULL) 
		    if(!request.Is_null())
			System.out.println("ERROR in MPI_Request_free, request not set to NULL\n");
		    //MPI_Recv(&datain,1,MPI_INT,0,0,comm,&status);
		    status = comm.Recv(datain,0,1,MPI.INT,0,0);
		}
		dataout[0] = ITER-1;
		//MPI_Send(&dataout,1,MPI_INT,0,0,comm);
		comm.Send(dataout,0,1,MPI.INT,0,0);
	    }

	    //MPI_Errhandler_set(MPI_COMM_WORLD,MPI_ERRORS_RETURN);
	    MPI.COMM_WORLD.Errhandler_set(MPI.ERRORS_RETURN) ;
	    /*
	    try {
	    }
	    catch(Exception e) {
	    }
	    */

	    /* The standard allows MPI_Request_free; in fact, 
	       this test MUST be successful (see the discussion 
	       of MPI_Request_free in the standard)
	       In fact, the example makes the use that is tested here. */
	    //MPI_Irecv(&datain,1,MPI_INT,0,0,MPI_COMM_WORLD,&request);
	    request = comm.Irecv(datain,0,1,MPI.INT,0,0);
	    //rc = MPI_Request_free(&request);
	    request.Free();
	    //MPI_Error_class(rc,&class);
	    // ???????????????????? 
	    if(class != MPI_SUCCESS)  
		System.out.println("ERROR: Attempt to free receive not allowed");

	    //MPI_Recv_init(&dataout,1,MPI_INT,0,0,MPI_COMM_WORLD,&request);
	    request = MPI.COMM_WORLD.Recv_init(dataout,0,1,MPI.INT,0,0);

	    //rc = MPI_Request_free(&request);
	    request.Free();
	    if(rc != MPI_SUCCESS)  
		System.out.println("ERROR: not able to free inactive receive request");

	    //MPI_Recv_init(&dataout,1,MPI_INT,0,0,MPI_COMM_WORLD,&request);
	    request = MPI.COMM_WORLD.Recv_init(dataout,0,1,MPI.INT,0,0);
	    //MPI_Start(&request);
	    request.Start();
	    //rc = MPI_Request_free(&request);
	    request.Free();
	    //MPI_Error_class(rc,&class);
	    // ???????????????????? 

	    if(class != MPI_SUCCESS)  
		System.out.println("ERROR: Attempt to free active persistent receive not allowed\n");

	}
	//MPI_Barrier(comm);
	comm.Barrier();
	if(me == 0)  System.out.println("Free TEST COMPLETE\n");
	MPI.Finalize();
    }
}




